#include "chil_global.h"

int
CHILSeekRaster(ChilboltonFile * chilfile,uword required_raster)
{
  unsigned short int i=0;
  if (! chilfile->summary_exists) {
    CHILPrintf("Summary file does not exist so cannot seek rasters\n");
    return CHIL_ERROR;
  }
  else {
    while (chilfile->raster[i] != required_raster && chilfile->nrasters > i++);
    if (chilfile->raster[i] == required_raster) {
      if (fseek(chilfile->file, CHIL_BLOCKSIZE*(chilfile->block[i]-1),
		SEEK_SET) == 0)
	{
	  chilfile->position = START_OF_IR;
	  chilfile->iraster = i;
	  chilfile->iscan = 0;
	  chilfile->bytes_read=0;
	  return CHIL_OK;
	}
      else
	{
	  chilfile->position = END_OF_DATA;
	  return CHIL_ERROR;
	}
    }
    else {
      return CHIL_NOTFOUND;
    }   
  }
}

/* Use a negative value for nscans to jump to the next raster */

int
CHILSkipScans(ChilboltonFile * chilfile, RasterInfo * info, int nscans)
{
  int *pscans = &nscans;
  return CHILSkipScansAndCount(chilfile, info, pscans);
}

int
CHILSkipScansAndCount(ChilboltonFile * chilfile, RasterInfo * info,
		      int *pscans)
{
  int iscan=0,ir=0;
  int nscans = *pscans;
  if (nscans != 0) {
    ir = (chilfile->position == START_OF_IR
	  || chilfile->position == START_OF_FILE);
    do {
      if (ir) {
	fseek(chilfile->file, 64, SEEK_CUR);
	if (feof(chilfile->file)) {
	  chilfile->position = END_OF_DATA;
	  CHILPrintf("End of file reached\n");
	  return CHIL_ERROR;
	}
	(chilfile->bytes_read) += 64;
	chilfile->position = START_OF_DATA;
      }
      else
	ir=1;
      while (CHILSkipRay(chilfile,info) == START_OF_RAY);
      iscan++;
    } while ((nscans < 0 || iscan < nscans) && chilfile->iscan > 0);
    if (chilfile->iscan == 0 && nscans > 0) {
      return CHIL_ERROR;
    }
  }
  *pscans = iscan;
  return CHIL_OK;
}

int
CHILSkipRay(ChilboltonFile * chilfile,RasterInfo * info)
{
  unsigned char signal[2];
  unsigned short nwords;

  if (chilfile->position !=START_OF_RAY
      && chilfile->position != START_OF_DATA) {
    CHILPrintf("Attempt to skip over a ray when at an inappropriate position in file\n");
    return END_OF_DATA;
  }
  else {
    fread(signal,sizeof(char),2,chilfile->file);
    (chilfile->bytes_read) += 2;
    nwords=signal[1] | signal[0] << 8;
    if (nwords > CHIL_RAYBUFFERSIZE) {
      switch (nwords) {
      case END_OF_SCAN:
	++(chilfile->iscan);
	return (chilfile->position = START_OF_IR);
	break;
      case END_OF_RASTER:case RASTER_ABORTED:
	return (CHILNextRaster(chilfile,info));
	break;
      default: /* Raw file? */
	return (chilfile->position = END_OF_DATA);
      }
    }
    if (fseek(chilfile->file,(nwords-1)*2,SEEK_CUR) == 0) {
      (chilfile->bytes_read) += (nwords-1)*2;
      return (chilfile->position = START_OF_RAY);
    }
    else {
      return (chilfile->position = END_OF_DATA);
    }
  }
}

int
CHILNextRaster(ChilboltonFile * chilfile,RasterInfo * info)
{
  int pad_size;

  pad_size = CHIL_BLOCKSIZE - chilfile->bytes_read % CHIL_BLOCKSIZE;
  /* Slight fudge required: it seems that some files have empty blocks
     of 4092 bytes, while others start the next raster immediately */
  if (pad_size < CHIL_BLOCKSIZE) {
    if (fseek(chilfile->file, pad_size, SEEK_CUR) != 0) {
      return (chilfile->position = END_OF_DATA);
    }
  }
  else {
    /* Is the next byte 255? This would indicate the start of an
       information record */
    unsigned char ir_start;
    if (fread(&ir_start, 1, 1, chilfile->file)) {
      ungetc(ir_start, chilfile->file);
      if (ir_start != 255) {
	/* Not an information record: jump forward one block */
	fseek(chilfile->file, CHIL_BLOCKSIZE, SEEK_CUR);
      }
    }
    else {
      return (chilfile->position = END_OF_DATA);
    }
  }
  chilfile->bytes_read = 0;
  (chilfile->iraster)++;
  chilfile->iscan = 0;
  return (chilfile->position = START_OF_IR);
}


